@@ -1,5 +1,8 @@
 # Copyright 2019 Michael Still
 
+from etcd3gw.client import Etcd3Client
+from etcd3gw.exceptions import ConnectionFailedError
+import json
 import socket
 
 from pydantic import (
@@ -14,6 +17,22 @@ def get_node_name():
     return socket.getfqdn()
 
 
+def etcd_settings(_settings):
+    try:
+        value = Etcd3Client(
+            host='localhost', port=2379, protocol='http', api_path='/v3beta/').get(
+                '/sf/config', metadata=True)
+        if value is None or len(value) == 0:
+            return {}
+        return json.loads(value[0][0])
+
+    except ConnectionFailedError:
+        # NOTE(mikal): I'm not sure this is the right approach, as it might cause
+        # us to silently ignore config errors. However, I can't just mock this away
+        # in tests because this code runs before the mocking occurs.
+        return {}
+
+
 class SFConfig(BaseSettings):
     ###################
     # Deployment Wide #
@@ -47,7 +66,8 @@ class SFConfig(BaseSettings):
         (
             '/srv/shakenfist/venv/bin/gunicorn --workers %(workers)d --bind 0.0.0.0:%(port)d '
             '--log-syslog --log-syslog-prefix sf --timeout %(timeout)s --name "%(name)s" '
-            '--pid /var/run/sf/gunicorn.pid shakenfist.external_api.app:app'
+            '--pid /var/run/sf/gunicorn.pid --worker-class gevent '
+            'shakenfist.external_api.app:app'
         ),
         description='The gunicorn command line to use'
     )
@@ -166,6 +186,10 @@ class SFConfig(BaseSettings):
         1800, description='How long to wait for an image transfer to occur before giving up'
     )
 
+    COMPRESS_SNAPSHOTS: bool = Field(
+        True, description='Compress snapshots taken of instances'
+    )
+
     # Artifact options
     ARTIFACT_MAX_VERSIONS_DEFAULT: int = Field(
         3, description='Default number of versions to keep within an artifact'
@@ -247,12 +271,12 @@ class SFConfig(BaseSettings):
         False, description='Add method name and module line number to log messages'
     )
 
-    EXCESSIVE_ETCD_CACHE_LOGGING: bool = Field(
-        False, description='Record detailed information about etcd cache performance.'
-    )
-
     class Config:
         env_prefix = 'SHAKENFIST_'
 
+        @classmethod
+        def customise_sources(cls, init_settings, env_settings, file_secret_settings):
+            return init_settings, etcd_settings, env_settings, file_secret_settings
+
 
 config = SFConfig()
