@@ -11,6 +11,7 @@
 
 import flask
 from flask_jwt_extended import JWTManager
+from flask_request_id import RequestID
 import flask_restful
 
 from shakenfist.config import config
@@ -36,6 +37,7 @@ daemon.set_log_level(LOG, 'api')
 
 
 app = flask.Flask(__name__)
+RequestID(app)
 api = flask_restful.Api(app, catch_all_404s=False)
 app.config['JWT_SECRET_KEY'] = config.AUTH_SECRET_SEED.get_secret_value()
 jwt = JWTManager(app)
@@ -46,10 +48,26 @@ app.logger.handlers = [HANDLER]
 
 @app.before_request
 def log_request_info():
-    LOG.debug(
-        'API request headers:\n' +
-        ''.join(['    %s: %s\n' % (h, v) for h, v in flask.request.headers]) +
-        'API request body: %s' % flask.request.get_data())
+    LOG.with_fields(
+        {
+            'request-id': flask.request.environ.get('FLASK_REQUEST_ID', 'none'),
+            'headers': flask.request.headers,
+            'body': flask.request.get_data()
+        }).debug('API request received')
+
+
+@app.after_request
+def log_response_info(response):
+    # Unfortunately the response body is too long to log here, but may be
+    # obtained with flask.response.get_data() if we ever want to grow a more
+    # complete tracing system.
+    log = LOG.with_fields(
+        {
+            'request-id': flask.request.environ.get('FLASK_REQUEST_ID', 'none'),
+            'headers': response.headers
+        })
+    log.debug('API response sent')
+    return response
 
 
 class Root(api_base.Resource):
@@ -61,6 +79,9 @@ class Root(api_base.Resource):
         return resp
 
 
+# TODO(mikal): we are inconsistent in this interface. Elsewhere the object type
+# is always signular, here its a mix. We should move all of these to the
+# singular form for consistency.
 api.add_resource(Root, '/')
 
 api.add_resource(api_admin.AdminLocksEndpoint, '/admin/locks')
@@ -78,35 +99,35 @@ api.add_resource(api_auth.AuthMetadatasEndpoint,
 api.add_resource(api_auth.AuthMetadataEndpoint,
                  '/auth/namespaces/<namespace>/metadata/<key>')
 
+api.add_resource(api_blob.BlobsEndpoint, '/blob')
 api.add_resource(api_blob.BlobEndpoint, '/blob/<blob_uuid>')
 
 api.add_resource(api_instance.InstancesEndpoint, '/instances')
-api.add_resource(api_instance.InstanceEndpoint, '/instances/<instance_uuid>')
+api.add_resource(api_instance.InstanceEndpoint, '/instances/<instance_ref>')
 api.add_resource(api_instance.InstanceEventsEndpoint,
-                 '/instances/<instance_uuid>/events')
+                 '/instances/<instance_ref>/events')
 api.add_resource(api_instance.InstanceInterfacesEndpoint,
-                 '/instances/<instance_uuid>/interfaces')
+                 '/instances/<instance_ref>/interfaces')
 api.add_resource(api_snapshot.InstanceSnapshotEndpoint,
-                 '/instances/<instance_uuid>/snapshot')
+                 '/instances/<instance_ref>/snapshot')
 api.add_resource(api_instance.InstanceRebootSoftEndpoint,
-                 '/instances/<instance_uuid>/rebootsoft')
+                 '/instances/<instance_ref>/rebootsoft')
 api.add_resource(api_instance.InstanceRebootHardEndpoint,
-                 '/instances/<instance_uuid>/reboothard')
+                 '/instances/<instance_ref>/reboothard')
 api.add_resource(api_instance.InstancePowerOffEndpoint,
-                 '/instances/<instance_uuid>/poweroff')
+                 '/instances/<instance_ref>/poweroff')
 api.add_resource(api_instance.InstancePowerOnEndpoint,
-                 '/instances/<instance_uuid>/poweron')
+                 '/instances/<instance_ref>/poweron')
 api.add_resource(api_instance.InstancePauseEndpoint,
-                 '/instances/<instance_uuid>/pause')
+                 '/instances/<instance_ref>/pause')
 api.add_resource(api_instance.InstanceUnpauseEndpoint,
-                 '/instances/<instance_uuid>/unpause')
+                 '/instances/<instance_ref>/unpause')
 api.add_resource(api_instance.InstanceMetadatasEndpoint,
-                 '/instances/<instance_uuid>/metadata')
+                 '/instances/<instance_ref>/metadata')
 api.add_resource(api_instance.InstanceMetadataEndpoint,
-                 '/instances/<instance_uuid>/metadata/<key>')
+                 '/instances/<instance_ref>/metadata/<key>')
 api.add_resource(api_instance.InstanceConsoleDataEndpoint,
-                 '/instances/<instance_uuid>/consoledata',
-                 defaults={'length': 10240})
+                 '/instances/<instance_ref>/consoledata')
 
 api.add_resource(api_interface.InterfaceEndpoint,
                  '/interfaces/<interface_uuid>')
@@ -129,19 +150,21 @@ api.add_resource(api_artifact.ArtifactVersionEndpoint,
 api.add_resource(api_label.LabelEndpoint, '/label/<label_name>')
 
 api.add_resource(api_network.NetworksEndpoint, '/networks')
-api.add_resource(api_network.NetworkEndpoint, '/networks/<network_uuid>')
+api.add_resource(api_network.NetworkEndpoint, '/networks/<network_ref>')
 api.add_resource(api_network.NetworkEventsEndpoint,
-                 '/networks/<network_uuid>/events')
+                 '/networks/<network_ref>/events')
 api.add_resource(api_network.NetworkInterfacesEndpoint,
-                 '/networks/<network_uuid>/interfaces')
+                 '/networks/<network_ref>/interfaces')
 api.add_resource(api_network.NetworkMetadatasEndpoint,
-                 '/networks/<network_uuid>/metadata')
+                 '/networks/<network_ref>/metadata')
 api.add_resource(api_network.NetworkMetadataEndpoint,
-                 '/networks/<network_uuid>/metadata/<key>')
+                 '/networks/<network_ref>/metadata/<key>')
 api.add_resource(api_network.NetworkPingEndpoint,
-                 '/networks/<network_uuid>/ping/<address>')
+                 '/networks/<network_ref>/ping/<address>')
 
 api.add_resource(api_node.NodesEndpoint, '/nodes')
 
 api.add_resource(api_upload.UploadCreateEndpoint, '/upload')
 api.add_resource(api_upload.UploadDataEndpoint, '/upload/<upload_uuid>')
+api.add_resource(api_upload.UploadTruncateEndpoint,
+                 '/upload/<upload_uuid>/truncate/<offset>')
